@@ -3,7 +3,7 @@
 #' A function to identify which specimen lies closest to the estimated mean 
 #' shape for a set of aligned specimens.
 #' 
-#' Function takes an array of aligned specimens (such as made by \code{\link{gpagen}}, 
+#' Function takes a 3D array of aligned specimens (such as made by \code{\link{gpagen}}, 
 #' calculates the distance of each to the estimated mean shape, and returns the name and 
 #' address of the closest specimen. This function can be used
 #' to identify the specimen to be used by \code{\link{warpRefMesh}}. 
@@ -16,7 +16,7 @@
 #' @return Function returns the name and address of the specimen closest to the mean of the set of 
 #' aligned specimens.
 findMeanSpec <- function(A){
-  if (length(dim(A)) != 3) {
+  if(!is.array(A)) {
     stop("Data matrix not a 3D array (see 'arrayspecs').") }
   ref <- mshape(A)
   x <- two.d.array(A)
