@@ -5,9 +5,14 @@
 #'
 #' The function performs a Generalized Procrustes Analysis (GPA) on two-dimensional or three-dimensional
 #'  landmark coordinates. The analysis can be performed on fixed landmark points, semilandmarks on 
-#'  curves, semilandmarks on surfaces, or any combination. To include semilandmarks on curves, one 
-#'  must specify a matrix defining which landmarks are to be treated as semilandmarks using the "curves=" 
-#'  option. Likewise, to include semilandmarks 
+#'  curves, semilandmarks on surfaces, or any combination. If data are provided in the form of a 3D array, all
+#'  landmarks and semilandmarks are contained in this object. If this is the only component provided, the function
+#'  will treat all points as if they were fixed landmarks. To designate some points as semilandmarks, one uses 
+#'  the "curves=" or "surfaces=" options (or both). To include semilandmarks on curves, a matrix defining 
+#'  which landmarks are to be treated as semilandmarks is provided using the "curves=" option. This matrix contains
+#'  three columns that specify the semilandmarks and two neighboring landmarks which are used to specify the tangent 
+#'  direction for sliding. The matrix may be generated using the function \code{\link{define.sliders}}). Likewise, 
+#'  to include semilandmarks 
 #'  on surfaces, one must specify a vector listing which landmarks are to be treated as surface semilandmarks 
 #'  using the "surfaces=" option. The "ProcD=TRUE" option will slide the semilandmarks along their tangent 
 #'  directions using the Procrustes distance criterion, while "ProcD=FALSE" will slide the semilandmarks 
@@ -108,6 +113,9 @@
 #' 
 #' # Example 2: points and semilandmarks on curves
 #' data(hummingbirds)
+#' 
+#' ###Slider matrix
+#' hummingbirds$curvepts
 #'
 #' # Using Procrustes Distance for sliding
 #' Y.gpa <- gpagen(hummingbirds$land,curves=hummingbirds$curvepts)   
@@ -129,10 +137,23 @@
 gpagen = function(A, curves=NULL, surfaces=NULL, PrinAxes = TRUE, 
                   max.iter = NULL, ProcD=TRUE, Proj = TRUE,
                   print.progress = TRUE){
-  n <- dim(A)[[3]]; p <- dim(A)[[1]]; k <- dim(A)[[2]]
-  if(!is.array(A)) stop("Coordinates must be a 3D array")
-  if(length(dim(A)) != 3) stop("Coordinates array does not have proper dimensions")
-  Y <- lapply(1:n, function(j) A[,,j])
+  if(inherits(A, "geomorphShapes")) {
+    Y <- A$landmarks
+    if(any(unlist(lapply(Y, is.na)))) stop("Data matrix contains missing values. Estimate these first (see 'estimate.missing').")
+    curves <- A$curves
+    n <- A$n
+    p <- A$p
+    k <- A$k
+    
+  } else {
+    
+    if(!is.array(A)) stop("Coordinates must be a 3D array")
+    if(length(dim(A)) != 3) stop("Coordinates array does not have proper dimensions")
+    if(any(is.na(A))) stop("Data matrix contains missing values. Estimate these first (see 'estimate.missing').")
+    n <- dim(A)[[3]]; p <- dim(A)[[1]]; k <- dim(A)[[2]]
+    Y <- lapply(1:n, function(j) A[,,j])
+  }
+  
   if(!is.logical(ProcD)) prD <- TRUE else prD <- ProcD
   if(is.null(max.iter)) max.it <- 5 else max.it <- as.numeric(max.iter)
   if(is.numeric(max.it) & max.it > 50) {
@@ -171,7 +192,8 @@ gpagen = function(A, curves=NULL, surfaces=NULL, PrinAxes = TRUE,
   iter <- gpa$iter
   pt.var <- Reduce("+",Map(function(y) y^2/n, coords))
   coords <- simplify2array(coords)
-  dimnames(coords)<- dimnames(A)
+  if(inherits(A, "geomorphShapes")) dimnames(coords)[[3]] <- names(A$landmarks) else
+    dimnames(coords) <- dimnames(A)
   pt.VCV <- var(two.d.array(coords))
   rownames(pt.var) <- dimnames(coords)[[1]]
   colnames(pt.var) <- if(k==3) c("Var.X", "Var.Y", "Var.Z") else 
